import sys
from PyQt5 import QtWidgets
from ewmh import EWMH
from gonha.threads import *
from gonha.util import Weather
from gonha.util import Smart
from gonha.util import Nvidia
from gonha.util import Radeon
from gonha.util import Language
from country_list import countries_for_language
import logging
import coloredlogs
import re

logger = logging.getLogger(__name__)
coloredlogs.install()


class Alert(QtWidgets.QDialog):
    lang = Language()

    def __init__(self, message):
        super(Alert, self).__init__()

        self.setWindowTitle(f"Gonha - [ {self.lang.getStr('alert')} ]")

        self.buttonBox = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.accepted.connect(self.accept)

        messageLabel = QtWidgets.QLabel(message)

        self.layout = QtWidgets.QVBoxLayout()
        self.layout.addWidget(messageLabel)
        self.layout.addWidget(self.buttonBox)
        self.setLayout(self.layout)


# noinspection PyTypeChecker
class MainWindow(QtWidgets.QMainWindow):
    config = Config()
    lang = Language()
    iface = config.getConfig('iface')
    version = config.getVersion()
    app = QtWidgets.QApplication(sys.argv)
    threadNetworkStats = ThreadNetworkStats()
    threadNvidia = ThreadNvidia()
    threadRadeon = ThreadRadeon()
    threadFast = ThreadFast()
    threadSlow = ThreadSlow()
    threadweather = ThreadWeather()
    partitionsWidgets = []
    upDownRateWidgets = []
    diskWidgets = []
    dtwWidgets = dict()
    systemWidgets = dict()
    nvidiaWidgets = list()
    radeonWidgets = list()
    verticalLayout = QtWidgets.QVBoxLayout()
    weather = Weather()
    debugRed = 'background-color: rgb(255, 48, 79);'
    nvidia = Nvidia()
    radeon = Radeon()
    smart = Smart()
    pbDefaultHeight = 20
    tempLabelWidth = 50

    def __init__(self):
        super(MainWindow, self).__init__()
        logger.info(self.lang.getStr('start'))
        # -------------------------------------------------------------
        # Window Flags
        flags = QtCore.Qt.FramelessWindowHint
        flags |= QtCore.Qt.WindowStaysOnBottomHint
        flags |= QtCore.Qt.Tool
        # -------------------------------------------------------------
        # Styles
        self.groupBoxStyle = """
        QGroupBox {
            border: 1px solid white;
            border-radius: 5px;
            margin-top: 12px;
            padding-left: 2px;
        }
        QGroupBox:title {
            subcontrol-origin: margin;
            subcontrol-position: top left;
            color: rgb(252, 126, 0);
            left: 15px;
        }
        """
        self.yellowPBStyle = """
        QProgressBar {
            text-align: left;
            font-weight: bold;
            color: rgb(255, 255, 255);
            background-color : rgba(0, 0, 0, 0);
            border: 0px solid rgba(0, 0, 0, 0);
            border-radius: 3px;                                    
        }
        QProgressBar::chunk {
            background: rgb(255, 153, 0);
            border-radius: 3px;            
        }
        """

        self.redPBStyle = """
        QProgressBar {
            text-align: left;
            font-weight: bold;
            color: rgb(255, 255, 255);
            background-color : rgba(0, 0, 0, 0);
            border: 0px solid rgba(0, 0, 0, 0);
            border-radius: 3px;                                    
        }
        QProgressBar::chunk {
            background: rgb(255, 51, 0);
            border-radius: 3px;            
        }
        """
        self.greenPBStyle = """
        QProgressBar {
            text-align: left;
            font-weight: bold;
            color: rgb(255, 255, 255);
            background-color : rgba(0, 0, 0, 0);
            border: 0px solid rgba(0, 0, 0, 0);
            border-radius: 3px;           
        }
        QProgressBar::chunk {
            background: rgb(51, 153, 51);
            border-radius: 3px;            
        }
        """
        self.orange = 'color: rgb(252, 126, 0);'
        self.white = 'color: rgb(255, 255, 255);'
        self.green = 'color: rgb(34, 255, 19);'
        self.red = 'color: rgb(255, 51, 0);'
        self.yellow = 'color: rgb(255, 153, 0);'

        # ---------------------------------------------------------------------
        # Default font
        self.fontDefault = QtGui.QFont('Fira Code', 11)
        self.fontGroupBox = QtGui.QFont('Fira Code', 14)
        self.groupBoxDefaultWidth = 450
        # -------------------------------------------------------------
        self.verticalLayout.setAlignment(QtCore.Qt.AlignTop)
        # --------------------------------------------------------------
        self.setWindowFlags(flags)
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)
        # Connect Threads Signals
        self.threadFast.signal.connect(self.receiveThreadFastfinish)
        self.threadSlow.signal.connect(self.receiveThreadSlowFinish)
        self.threadNetworkStats.signal.connect(self.receiveThreadNetworkStats)
        self.threadweather.signal.connect(self.receiveThreadWeatherFinish)
        self.threadNvidia.signal.connect(self.receiveThreadNvidia)
        self.threadRadeon.signal.connect(self.receiveThreadRadeon)
        centralWidGet = QtWidgets.QWidget(self)
        centralWidGet.setLayout(self.verticalLayout)
        self.setCentralWidget(centralWidGet)
        # -----------------------------------------------------------------------------------------------
        # before show main window, check all dependencies and refuse if any thing wrong
        self.checkDependencies()
        #
        self.show()
        # Show in all workspaces
        self.ew = EWMH()
        self.all_wins = self.ew.getClientList()
        self.wins = filter(lambda wHandle: wHandle.get_wm_class()[1] == 'gonha', self.all_wins)
        for w in self.wins:
            self.ew.setWmDesktop(w, 0xffffffff)

        self.ew.display.flush()

        self.threadFast.start()
        self.threadSlow.start()
        self.threadNetworkStats.start()

        self.loadPosition()
        self.displayDTWeather()
        self.displaySystem()
        if self.nvidia.getSmiStatus():
            if self.nvidia.nvidiaEntity['count'] >= 0:
                self.displayNvidia()

        if self.radeon.has_radeon():
            self.displayRadeon()
        self.displayIface()
        self.displayPartitions()

    def checkDependencies(self):
        # ------------------------------------------------------------------------
        # Check if user need update config file
        oldVersion = self.config.getConfig('version')
        if self.config.getVersion() != oldVersion:
            dialog = Alert(self.lang.getStr('runconfig'))
            dialog.exec_()
            sys.exit()
        # ------------------------------------------------------------------------
        # check for hddtemp
        if not self.smart.hddtempIsOk():
            dialog = Alert(self.lang.getStr('hddtemprequire'))
            dialog.exec_()
            sys.exit()

        # Check Kernel dependencies only run in kernel >=5.5
        kernel = self.config.getKernelInfo()
        if not ((kernel['kernelVersion'] >= 5) and (kernel['majorRevision'] >= 5)):
            dialog = Alert(self.lang.getStr('kernelrequire'))
            dialog.exec_()
            sys.exit()

    def getDefaultGb(self, title):
        defaultGb = QtWidgets.QGroupBox(title)
        defaultGb.setFont(self.fontGroupBox)
        defaultGb.setStyleSheet(self.groupBoxStyle)
        defaultGb.setFixedWidth(self.groupBoxDefaultWidth)
        return defaultGb

    def displayRadeon(self):
        radeonGroupBox = self.getDefaultGb('radeon')
        gridLayout = QtWidgets.QGridLayout()
        gridLayout.setVerticalSpacing(1)
        gridLayout.setHorizontalSpacing(0)
        # ---------------------------------------------------
        # radeon data
        radeonLogoLabel = QtWidgets.QLabel()
        radeonLogoLabel.setPixmap(QtGui.QPixmap(f"{self.config.resource_path}/images/radeon.png"))
        radeonLogoLabel.setFixedSize(64, 57)
        radeonLogoLabel.setAlignment(QtCore.Qt.AlignTop)

        gridLayout.addWidget(radeonLogoLabel, 0, 0, -1, 1)

        for gpu in self.radeon.radeon_entity['gpus']:
            temp_dict = dict()

            model_label = QtWidgets.QLabel(f"{self.lang.getStr('model')}:")
            self.setLabel(model_label, self.orange, self.fontDefault)
            model_label.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

            gridLayout.addWidget(model_label, 0, 1)

            model_value_label = QtWidgets.QLabel(gpu['gpu_name'])
            temp_dict['gpu_name'] = model_value_label
            model_value_label.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
            self.setLabel(model_value_label, self.white, self.fontDefault)

            gridLayout.addWidget(model_value_label, 0, 2)

            memory_label = QtWidgets.QLabel(f"{self.lang.getStr('memory')}:")
            memory_label.setFixedWidth(70)
            self.setLabel(memory_label, self.orange, self.fontDefault)
            memory_label.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

            gridLayout.addWidget(memory_label, 1, 1)

            used_total_mem_label = QtWidgets.QLabel(
                f"{round(gpu['memory_used'] / 1e+9, 2)}GiB/{round(gpu['memory_info']['vram_size'] / 1e+9, 2)}GiB")
            temp_dict['usedTotalMemory'] = used_total_mem_label
            self.setLabel(used_total_mem_label, self.white, self.fontDefault)
            used_total_mem_label.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)

            gridLayout.addWidget(used_total_mem_label, 1, 2)

            temp_icon = QtWidgets.QLabel()
            temp_icon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/temp.png'))
            temp_icon.setAlignment(QtCore.Qt.AlignHCenter | QtCore.Qt.AlignVCenter)
            temp_icon.setFixedSize(24, 24)

            gridLayout.addWidget(temp_icon, 1, 3)

            temp_label = QtWidgets.QLabel('')
            temp_dict['temperature_gpu'] = temp_label
            temp_label.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            self.setLabel(temp_label, self.white, self.fontDefault)
            temp_label.setFixedWidth(80)

            gridLayout.addWidget(temp_label, 1, 4)

            self.radeonWidgets.append(temp_dict)

        radeonGroupBox.setLayout(gridLayout)
        self.verticalLayout.addWidget(radeonGroupBox)
        self.threadRadeon.start()

    def displayNvidia(self):
        nvidiaGroupBox = self.getDefaultGb('nvidia')
        gridLayout = QtWidgets.QGridLayout()
        gridLayout.setVerticalSpacing(1)
        gridLayout.setHorizontalSpacing(0)
        # ---------------------------------------------------
        # nvidia data

        nvidiaLogoLabel = QtWidgets.QLabel()
        nvidiaLogoLabel.setPixmap(QtGui.QPixmap(f"{self.config.resource_path}/images/nvidia.png"))
        nvidiaLogoLabel.setFixedSize(64, 57)
        nvidiaLogoLabel.setAlignment(QtCore.Qt.AlignTop)

        gridLayout.addWidget(nvidiaLogoLabel, 0, 0, -1, 1)

        for gpu in self.nvidia.nvidiaEntity['gpus']:
            tempDict = dict()

            modelLabel = QtWidgets.QLabel(f"{self.lang.getStr('model')}:")
            self.setLabel(modelLabel, self.orange, self.fontDefault)
            # noinspection PyTypeChecker
            modelLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

            gridLayout.addWidget(modelLabel, 0, 1)

            modelValueLabel = QtWidgets.QLabel(gpu['gpu_name'])
            tempDict['gpu_name'] = modelValueLabel
            modelValueLabel.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
            self.setLabel(modelValueLabel, self.white, self.fontDefault)

            gridLayout.addWidget(modelValueLabel, 0, 2)

            loadIcon = QtWidgets.QLabel()
            loadIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/load.png'))
            loadIcon.setAlignment(QtCore.Qt.AlignHCenter | QtCore.Qt.AlignVCenter)
            loadIcon.setFixedSize(24, 24)

            gridLayout.addWidget(loadIcon, 0, 3)

            loadValueLabel = QtWidgets.QLabel(gpu['utilization_gpu'])
            tempDict['utilization_gpu'] = loadValueLabel
            loadValueLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            loadValueLabel.setFixedWidth(80)
            self.setLabel(loadValueLabel, self.white, self.fontDefault)

            gridLayout.addWidget(loadValueLabel, 0, 4)

            memoryLabel = QtWidgets.QLabel(f"{self.lang.getStr('memory')}:")
            memoryLabel.setFixedWidth(70)
            self.setLabel(memoryLabel, self.orange, self.fontDefault)
            memoryLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

            gridLayout.addWidget(memoryLabel, 1, 1)

            usedTotalMemLabel = QtWidgets.QLabel(f"{gpu['memory_used']}/{gpu['memory_total']}")
            tempDict['usedTotalMemory'] = usedTotalMemLabel
            self.setLabel(usedTotalMemLabel, self.white, self.fontDefault)
            usedTotalMemLabel.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)

            gridLayout.addWidget(usedTotalMemLabel, 1, 2)

            tempIcon = QtWidgets.QLabel()
            tempIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/temp.png'))
            tempIcon.setAlignment(QtCore.Qt.AlignHCenter | QtCore.Qt.AlignVCenter)
            tempIcon.setFixedSize(24, 24)

            gridLayout.addWidget(tempIcon, 1, 3)

            tempLabel = QtWidgets.QLabel('')
            tempDict['temperature_gpu'] = tempLabel
            tempLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            self.setLabel(tempLabel, self.white, self.fontDefault)
            tempLabel.setFixedWidth(80)

            gridLayout.addWidget(tempLabel, 1, 4)

            # Driver Version

            driverLabel = QtWidgets.QLabel(f"{self.lang.getStr('driver')}:")
            self.setLabel(driverLabel, self.orange, self.fontDefault)
            driverLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

            gridLayout.addWidget(driverLabel, 2, 1)

            driverValueLabel = QtWidgets.QLabel(self.nvidia.nvidiaEntity['driver_version'])
            self.setLabel(driverValueLabel, self.white, self.fontDefault)
            driverValueLabel.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)

            gridLayout.addWidget(driverValueLabel, 2, 2)

            fanIcon = QtWidgets.QLabel()
            fanIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/fan.png'))
            fanIcon.setAlignment(QtCore.Qt.AlignHCenter)
            fanIcon.setFixedSize(24, 24)

            gridLayout.addWidget(fanIcon, 2, 3)

            fanValueLabel = QtWidgets.QLabel(gpu['fan_speed'])
            self.setLabel(fanValueLabel, self.white, self.fontDefault)
            fanValueLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            fanValueLabel.setFixedWidth(80)

            tempDict['fan_speed'] = fanValueLabel

            gridLayout.addWidget(fanValueLabel, 2, 4)

            # bios
            biosLabel = QtWidgets.QLabel(f"{self.lang.getStr('bios')}:")
            self.setLabel(biosLabel, self.orange, self.fontDefault)
            biosLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

            gridLayout.addWidget(biosLabel, 3, 1)

            biosValueLabel = QtWidgets.QLabel(gpu['vbios_version'])
            self.setLabel(biosValueLabel, self.white, self.fontDefault)
            biosValueLabel.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)

            gridLayout.addWidget(biosValueLabel, 3, 2)

            powerIcon = QtWidgets.QLabel()
            powerIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/power.png'))
            powerIcon.setAlignment(QtCore.Qt.AlignHCenter | QtCore.Qt.AlignVCenter)
            powerIcon.setFixedSize(24, 24)

            gridLayout.addWidget(powerIcon, 3, 3)

            powerDrawLabel = QtWidgets.QLabel(gpu['power_draw'])
            self.setLabel(powerDrawLabel, self.white, self.fontDefault)
            powerDrawLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            powerDrawLabel.setFixedWidth(80)
            tempDict['power_draw'] = powerDrawLabel

            gridLayout.addWidget(powerDrawLabel, 3, 4)

            self.nvidiaWidgets.append(tempDict)

        nvidiaGroupBox.setLayout(gridLayout)
        self.verticalLayout.addWidget(nvidiaGroupBox)
        self.threadNvidia.start()

    def displayIface(self):
        verticalLayout = QtWidgets.QVBoxLayout()
        ifaceGroupBox = self.getDefaultGb(self.lang.getStr('net'))
        rateLabelWidth = 120
        # ---------------------------------------------------
        # Ip int Label

        ipLayout = QtWidgets.QGridLayout()

        intipLabel = QtWidgets.QLabel(f"{self.lang.getStr('intabbvr')}:")
        self.setLabel(intipLabel, self.orange, self.fontDefault)

        ipLayout.addWidget(intipLabel, 0, 0)

        # ip int value label
        intipValueLabel = QtWidgets.QLabel('')
        self.setLabel(intipValueLabel, self.white, self.fontDefault)
        self.systemWidgets['intip'] = intipValueLabel

        ipLayout.addWidget(intipValueLabel, 0, 1)

        # Ext Ip
        extipLabel = QtWidgets.QLabel(f"{self.lang.getStr('extabbvr')}:")
        self.setLabel(extipLabel, self.orange, self.fontDefault)

        ipLayout.addWidget(extipLabel, 0, 2)

        extipValueLabel = QtWidgets.QLabel('')
        self.setLabel(extipValueLabel, self.white, self.fontDefault)
        self.systemWidgets['extip'] = extipValueLabel

        ipLayout.addWidget(extipValueLabel, 0, 3)

        verticalLayout.addLayout(ipLayout)

        # -------------------------------------------------

        netGridLayout = QtWidgets.QGridLayout()

        netCardIcon = QtWidgets.QLabel()
        netCardIcon.setPixmap(QtGui.QPixmap(f"{self.config.resource_path}/images/netcard.png"))
        netCardIcon.setFixedSize(24, 24)

        netGridLayout.addWidget(netCardIcon, 0, 0)

        # -------------------------------------------------
        # ifaceValueLabel
        ifaceValueLabel = QtWidgets.QLabel('')
        self.setLabel(ifaceValueLabel, self.white, self.fontDefault)
        self.upDownRateWidgets.append(ifaceValueLabel)
        ifaceValueLabel.setFixedWidth(80)

        netGridLayout.addWidget(ifaceValueLabel, 0, 1)

        # -------------------------------------------------
        # Download Icon
        downloadIcon = QtWidgets.QLabel()
        downloadIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/download.png'))
        downloadIcon.setFixedSize(24, 24)

        netGridLayout.addWidget(downloadIcon, 0, 2)

        # ---------------------------------------------------
        # download rate label
        ifaceDownRateLabel = QtWidgets.QLabel('')
        self.setLabel(ifaceDownRateLabel, self.white, self.fontDefault)
        self.upDownRateWidgets.append(ifaceDownRateLabel)
        ifaceDownRateLabel.setFixedWidth(rateLabelWidth)
        ifaceDownRateLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        netGridLayout.addWidget(ifaceDownRateLabel, 0, 3)
        # ---------------------------------------------------

        # -------------------------------------------------
        # Upload Icon
        uploadIcon = QtWidgets.QLabel()
        uploadIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/upload.png'))
        uploadIcon.setFixedSize(24, 24)

        netGridLayout.addWidget(uploadIcon, 0, 4)
        # -------------------------------------------------

        # ---------------------------------------------------
        # upload rate label
        ifaceUpRateLabel = QtWidgets.QLabel('')
        self.setLabel(ifaceUpRateLabel, self.white, self.fontDefault)
        self.upDownRateWidgets.append(ifaceUpRateLabel)
        ifaceUpRateLabel.setFixedWidth(rateLabelWidth)
        ifaceUpRateLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        netGridLayout.addWidget(ifaceUpRateLabel, 0, 5)

        verticalLayout.addLayout(netGridLayout)

        # ---------------------------------------------------

        # Total in

        bytesLayout = QtWidgets.QGridLayout()

        bytesRcvLabel = QtWidgets.QLabel(f"{self.lang.getStr('totalin')}:")
        self.setLabel(bytesRcvLabel, self.orange, self.fontDefault)

        bytesLayout.addWidget(bytesRcvLabel, 0, 0)

        bytesRcvValueLabel = QtWidgets.QLabel('')
        self.setLabel(bytesRcvValueLabel, self.white, self.fontDefault)
        self.upDownRateWidgets.append(bytesRcvValueLabel)

        bytesLayout.addWidget(bytesRcvValueLabel, 0, 1)

        # Total out
        bytesSentLabel = QtWidgets.QLabel(f"{self.lang.getStr('totalout')}:")
        self.setLabel(bytesSentLabel, self.orange, self.fontDefault)

        bytesLayout.addWidget(bytesSentLabel, 0, 2)

        bytesSentValueLabel = QtWidgets.QLabel('')
        self.setLabel(bytesSentValueLabel, self.white, self.fontDefault)
        self.upDownRateWidgets.append(bytesSentValueLabel)

        bytesLayout.addWidget(bytesSentValueLabel, 0, 3)

        verticalLayout.addLayout(bytesLayout)

        ifaceGroupBox.setLayout(verticalLayout)
        self.verticalLayout.addWidget(ifaceGroupBox)

    def displayDTWeather(self):
        countries = dict(countries_for_language(self.lang.getLocalLanguage()))
        timeHeight = 50
        dateHeight = 25
        tempHeight = 60

        timeFont = QtGui.QFont('Fira Code', 45)
        dayFont = QtGui.QFont('Fira Code', 20)
        weekdayFont = QtGui.QFont('Fira Code', 15)
        yearFont = QtGui.QFont('Fira Code', 12)
        monthFont = QtGui.QFont('Fira Code', 12)

        gray = 'color: rgb(143, 143, 143);'

        mainHBLayout = QtWidgets.QHBoxLayout()
        mainHBLayout.setSpacing(0)
        mainHBLayout.setAlignment(QtCore.Qt.AlignHCenter)

        # Horizontal Layout for time
        timeHBLayout = QtWidgets.QHBoxLayout()
        timeHBLayout.setAlignment(QtCore.Qt.AlignHCenter)

        twoPointLabel = [QtWidgets.QLabel(':'), QtWidgets.QLabel(':')]
        for label in twoPointLabel:
            label.setFont(timeFont)
            label.setStyleSheet(gray)
            label.setFixedHeight(timeHeight)

        hourLabel = QtWidgets.QLabel('22')
        self.setLabel(hourLabel, self.white, timeFont)
        hourLabel.setFixedHeight(timeHeight)
        self.dtwWidgets['hour'] = hourLabel

        minLabel = QtWidgets.QLabel('24')
        self.setLabel(minLabel, self.white, timeFont)
        minLabel.setFixedHeight(timeHeight)
        self.dtwWidgets['min'] = minLabel

        timeHBLayout.addWidget(hourLabel)
        timeHBLayout.addWidget(twoPointLabel[0])
        timeHBLayout.addWidget(minLabel)

        self.dtwWidgets['hour'] = hourLabel
        self.dtwWidgets['min'] = minLabel

        mainHBLayout.addLayout(timeHBLayout)

        # date vertical layout
        dateVBLayout = QtWidgets.QVBoxLayout()
        # date horizontal layout
        dateHBLayout = QtWidgets.QHBoxLayout()
        dateHBLayout.setAlignment(QtCore.Qt.AlignLeft)

        dayLabel = QtWidgets.QLabel('05')
        self.setLabel(dayLabel, self.orange, dayFont)
        dayLabel.setFixedHeight(dateHeight)

        monthLabel = QtWidgets.QLabel('June')
        self.setLabel(monthLabel, self.orange, monthFont)
        monthLabel.setFixedHeight(dateHeight)
        monthLabel.setAlignment(QtCore.Qt.AlignBottom)

        yearLabel = QtWidgets.QLabel('2020')
        yearLabel.setFont(yearFont)
        yearLabel.setStyleSheet(self.white)
        self.setLabel(yearLabel, self.orange, yearFont)
        yearLabel.setFixedHeight(dateHeight)
        yearLabel.setAlignment(QtCore.Qt.AlignBottom)

        dateHBLayout.addWidget(dayLabel)
        dateHBLayout.addWidget(monthLabel)
        dateHBLayout.addWidget(yearLabel)
        self.dtwWidgets['day'] = dayLabel
        self.dtwWidgets['month'] = monthLabel
        self.dtwWidgets['year'] = yearLabel

        dateVBLayout.addLayout(dateHBLayout)

        weekdayHBLayout = QtWidgets.QHBoxLayout()

        weekdayLabel = QtWidgets.QLabel('Saturday')
        self.setLabel(weekdayLabel, self.orange, weekdayFont)
        weekdayLabel.setFixedHeight(20)

        weekdayHBLayout.addWidget(weekdayLabel)
        self.dtwWidgets['weekday'] = weekdayLabel

        dateVBLayout.addLayout(weekdayHBLayout)

        mainHBLayout.addLayout(dateVBLayout)

        # --------------------------------------------------------
        # weather conditions

        weatherHBLayout = QtWidgets.QHBoxLayout()

        weatherVBLayout = QtWidgets.QVBoxLayout()
        weatherVBLayout.setSpacing(0)
        weatherVBLayout.setAlignment(QtCore.Qt.AlignVCenter)

        tempLabel = QtWidgets.QLabel('')
        self.setLabel(tempLabel, self.white, timeFont)
        tempLabel.setFixedHeight(tempHeight)
        self.dtwWidgets['temp'] = tempLabel

        weatherHBLayout.addWidget(tempLabel)

        # Cloud Icon
        cloudIconLabel = QtWidgets.QLabel()
        cloudIconLabel.setFixedHeight(42)
        cloudIconLabel.setFixedHeight(tempHeight)
        self.dtwWidgets['cloudicon'] = cloudIconLabel

        weatherHBLayout.addWidget(cloudIconLabel)
        weatherHBLayout.setAlignment(QtCore.Qt.AlignHCenter)

        cityRegionLabel = QtWidgets.QLabel(
            f"{self.config.getConfig('location')['city']}")
        self.setLabel(cityRegionLabel, self.orange, self.fontDefault)

        countryLabel = QtWidgets.QLabel(countries[self.config.getConfig('location')['country']])
        self.setLabel(countryLabel, self.orange, self.fontDefault)

        weatherVBLayout.addWidget(cityRegionLabel)
        weatherVBLayout.addWidget(countryLabel)

        weatherHBLayout.addLayout(weatherVBLayout)
        # ---------------------------------------------------------------------
        # humidity, pressure, visibility,  wind,
        weatherGridLayout = QtWidgets.QGridLayout()
        weatherGridLayout.setSpacing(0)

        # humidityIcon
        humidityIcon = QtWidgets.QLabel()
        humidityIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/humidity.png'))
        humidityIcon.setFixedWidth(32)

        pressureIcon = QtWidgets.QLabel()
        pressureIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/pressure.png'))
        pressureIcon.setFixedWidth(32)

        visibilityIcon = QtWidgets.QLabel()
        visibilityIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/visibility.png'))
        visibilityIcon.setFixedWidth(32)

        windIcon = QtWidgets.QLabel()
        windIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/wind.png'))
        windIcon.setFixedWidth(32)

        weatherGridLayout.addWidget(humidityIcon, 0, 0, 1, 1, QtCore.Qt.AlignHCenter)
        weatherGridLayout.addWidget(pressureIcon, 0, 1, 1, 1, QtCore.Qt.AlignHCenter)
        weatherGridLayout.addWidget(visibilityIcon, 0, 2, 1, 1, QtCore.Qt.AlignHCenter)
        weatherGridLayout.addWidget(windIcon, 0, 3, 1, 1, QtCore.Qt.AlignHCenter)
        # ---------------------------------------------------------------------

        humidityLabel = QtWidgets.QLabel('')
        self.setLabel(humidityLabel, self.white, self.fontDefault)
        self.dtwWidgets['humidity'] = humidityLabel

        pressureLabel = QtWidgets.QLabel('')
        self.setLabel(pressureLabel, self.white, self.fontDefault)
        self.dtwWidgets['pressure'] = pressureLabel

        visibilityLabel = QtWidgets.QLabel('')
        self.setLabel(visibilityLabel, self.white, self.fontDefault)
        self.dtwWidgets['visibility'] = visibilityLabel

        windLabel = QtWidgets.QLabel('')
        self.setLabel(windLabel, self.white, self.fontDefault)
        self.dtwWidgets['wind'] = windLabel

        weatherGridLayout.addWidget(humidityLabel, 1, 0, 1, 1, QtCore.Qt.AlignHCenter)
        weatherGridLayout.addWidget(pressureLabel, 1, 1, 1, 1, QtCore.Qt.AlignHCenter)
        weatherGridLayout.addWidget(visibilityLabel, 1, 2, 1, 1, QtCore.Qt.AlignHCenter)
        weatherGridLayout.addWidget(windLabel, 1, 3, 1, 1, QtCore.Qt.AlignHCenter)

        self.verticalLayout.addLayout(mainHBLayout)
        self.verticalLayout.addLayout(weatherHBLayout)
        self.verticalLayout.addLayout(weatherGridLayout)
        self.threadweather.updateWeather()

    def displaySystem(self):
        pbDefaultWidth = 180
        iconDefaultWidth = 80
        systemGroupBox = self.getDefaultGb(f"{self.lang.getStr('system')}")

        verticalLayout = QtWidgets.QVBoxLayout()
        verticalLayout.setSpacing(0)

        distroGridLayout = QtWidgets.QGridLayout()
        distroGridLayout.setAlignment(QtCore.Qt.AlignVCenter | QtCore.Qt.AlignHCenter)
        distroGridLayout.setSpacing(0)

        # distro Label
        distroJson = self.config.getConfig('distro')
        distroStr = f"{distroJson['name']} {distroJson['version']}"
        if not distroJson['codename'] == '':
            distroStr = f"{distroStr} {distroJson['codename']}"

        # ---------------------------------------------------------------------------
        distroIcon = QtWidgets.QLabel()
        distroIcon.setPixmap(QtGui.QPixmap(distroJson['iconfile']))

        distroGridLayout.addWidget(distroIcon, 0, 0, -1, 1)

        # ---------------------------------------------------------------------------
        # Distro label
        distroLabel = QtWidgets.QLabel(distroStr)
        self.setLabel(distroLabel, self.white, self.fontDefault)

        distroGridLayout.addWidget(distroLabel, 0, 1)

        # ---------------------------------------------------------------------------
        # kernel label
        platJson = self.config.getConfig('platform')
        kernelLabel = QtWidgets.QLabel(f"Kernel {platJson['release']}")
        self.setLabel(kernelLabel, self.white, self.fontDefault)

        distroGridLayout.addWidget(kernelLabel, 1, 1)
        # ---------------------------------------------------------------------------
        # Machine Label
        machineLabel = QtWidgets.QLabel(f"{self.lang.getStr('node')} {platJson['node']} arch {platJson['machine']}")
        self.setLabel(machineLabel, self.white, self.fontDefault)

        distroGridLayout.addWidget(machineLabel, 2, 1)
        # ---------------------------------------------------------------------------

        # ---------------------------------------------------------------------------
        # boot time label
        verticalLayout.addLayout(distroGridLayout)

        bootAndCpuModelLayout = QtWidgets.QVBoxLayout()
        bootAndCpuModelLayout.setSpacing(0)
        btHLayout = QtWidgets.QHBoxLayout()
        btHLayout.setAlignment(QtCore.Qt.AlignHCenter | QtCore.Qt.AlignVCenter)

        # --------------------------------------------------------------------
        # days, hours, minutes and seconds labels

        btDaysLabel = QtWidgets.QLabel('d')
        self.setLabel(btDaysLabel, self.green, self.fontDefault)
        btHLayout.addWidget(btDaysLabel)
        self.systemWidgets['btDays'] = btDaysLabel

        daysLabel = QtWidgets.QLabel(f"{self.lang.getStr('days')}, ")
        self.setLabel(daysLabel, self.white, self.fontDefault)
        btHLayout.addWidget(daysLabel)

        btHoursLabel = QtWidgets.QLabel('h')
        self.setLabel(btHoursLabel, self.green, self.fontDefault)
        btHLayout.addWidget(btHoursLabel)
        self.systemWidgets['btHours'] = btHoursLabel

        hoursLabel = QtWidgets.QLabel(f"{self.lang.getStr('hours')} ")
        self.setLabel(hoursLabel, self.white, self.fontDefault)
        btHLayout.addWidget(hoursLabel)

        btMinutesLabel = QtWidgets.QLabel('m')
        self.setLabel(btMinutesLabel, self.green, self.fontDefault)
        btHLayout.addWidget(btMinutesLabel)
        self.systemWidgets['btMinutes'] = btMinutesLabel

        minutesLabel = QtWidgets.QLabel(f"{self.lang.getStr('minutes')} ")
        self.setLabel(minutesLabel, self.white, self.fontDefault)
        btHLayout.addWidget(minutesLabel)

        btSecondsLabel = QtWidgets.QLabel('s')
        self.setLabel(btSecondsLabel, self.green, self.fontDefault)
        btHLayout.addWidget(btSecondsLabel)
        self.systemWidgets['btSeconds'] = btSecondsLabel

        secondsLabel = QtWidgets.QLabel(f"{self.lang.getStr('seconds')}")
        self.setLabel(secondsLabel, self.white, self.fontDefault)
        btHLayout.addWidget(secondsLabel)

        bootAndCpuModelLayout.addLayout(btHLayout)

        # ---------------------------------------------------------------------------
        cpuBrandLabel = QtWidgets.QLabel(self.config.getConfig('cpuinfo'))
        self.setLabel(cpuBrandLabel, self.white, self.fontDefault)
        cpuBrandLabel.setAlignment(QtCore.Qt.AlignHCenter | QtCore.Qt.AlignVCenter)

        bootAndCpuModelLayout.addWidget(cpuBrandLabel)

        verticalLayout.addLayout(bootAndCpuModelLayout)

        # Cpu load

        gridLayout = QtWidgets.QGridLayout()
        cpuIcon = QtWidgets.QLabel()
        cpuIcon.setPixmap(QtGui.QPixmap(f"{self.config.resource_path}/images/cpu.png"))
        cpuIcon.setFixedWidth(iconDefaultWidth)

        gridLayout.addWidget(cpuIcon, 0, 0)

        cpuProgressBar = QtWidgets.QProgressBar()
        cpuProgressBar.setFixedHeight(self.pbDefaultHeight)
        cpuProgressBar.setFont(self.fontDefault)
        cpuProgressBar.setStyleSheet(self.greenPBStyle)
        cpuProgressBar.setValue(12)
        self.systemWidgets['cpuProgressBar'] = cpuProgressBar

        gridLayout.addWidget(cpuProgressBar, 0, 1)

        cpuFreqCurrentLabel = QtWidgets.QLabel('')
        self.setLabel(cpuFreqCurrentLabel, self.white, self.fontDefault)
        self.systemWidgets['cpuFreqCurrent'] = cpuFreqCurrentLabel
        cpuFreqCurrentLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        gridLayout.addWidget(cpuFreqCurrentLabel, 0, 2)

        cpuFreqSeparatorLabel = QtWidgets.QLabel('/')
        self.setLabel(cpuFreqSeparatorLabel, self.white, self.fontDefault)
        cpuFreqSeparatorLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        gridLayout.addWidget(cpuFreqSeparatorLabel, 0, 3)

        cpuFreqMaxLabel = QtWidgets.QLabel('')
        self.setLabel(cpuFreqMaxLabel, self.green, self.fontDefault)
        cpuFreqMaxLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.systemWidgets['cpuFreqMax'] = cpuFreqMaxLabel

        gridLayout.addWidget(cpuFreqMaxLabel, 0, 4)

        # ---------------------------------------------------------------------------
        # ram load

        ramIcon = QtWidgets.QLabel()
        ramIcon.setPixmap(QtGui.QPixmap(f"{self.config.resource_path}/images/ram.png"))

        gridLayout.addWidget(ramIcon, 1, 0)

        ramProgressBar = QtWidgets.QProgressBar()
        ramProgressBar.setFixedHeight(self.pbDefaultHeight)
        ramProgressBar.setFixedWidth(pbDefaultWidth)
        ramProgressBar.setFont(self.fontDefault)
        ramProgressBar.setStyleSheet(self.greenPBStyle)
        self.systemWidgets['ramProgressBar'] = ramProgressBar
        ramProgressBar.setValue(32)

        gridLayout.addWidget(ramProgressBar, 1, 1)

        ramUsedLabel = QtWidgets.QLabel('')
        self.setLabel(ramUsedLabel, self.white, self.fontDefault)
        self.systemWidgets['ramUsed'] = ramUsedLabel
        ramUsedLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        gridLayout.addWidget(ramUsedLabel, 1, 2)

        ramSeparatorLabel = QtWidgets.QLabel('/')
        self.setLabel(ramSeparatorLabel, self.white, self.fontDefault)
        ramSeparatorLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        gridLayout.addWidget(ramSeparatorLabel, 1, 3)

        ramTotalLabel = QtWidgets.QLabel('')
        self.setLabel(ramTotalLabel, self.green, self.fontDefault)
        self.systemWidgets['ramTotal'] = ramTotalLabel
        ramTotalLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        gridLayout.addWidget(ramTotalLabel, 1, 4)

        # ---------------------------------------------------------------------------
        # swap load

        swapIcon = QtWidgets.QLabel()
        swapIcon.setPixmap(QtGui.QPixmap(f"{self.config.resource_path}/images/swap.png"))

        gridLayout.addWidget(swapIcon, 2, 0)

        swapProgressBar = QtWidgets.QProgressBar()
        swapProgressBar.setFixedHeight(self.pbDefaultHeight)
        swapProgressBar.setFixedWidth(pbDefaultWidth)
        swapProgressBar.setFont(self.fontDefault)
        swapProgressBar.setStyleSheet(self.greenPBStyle)
        self.systemWidgets['swapProgressBar'] = swapProgressBar
        swapProgressBar.setValue(52)

        gridLayout.addWidget(swapProgressBar, 2, 1)

        swapUsedLabel = QtWidgets.QLabel('')
        self.setLabel(swapUsedLabel, self.white, self.fontDefault)
        self.systemWidgets['swapUsed'] = swapUsedLabel
        swapUsedLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        gridLayout.addWidget(swapUsedLabel, 2, 2)

        swapSeparatorLabel = QtWidgets.QLabel('/')
        self.setLabel(swapSeparatorLabel, self.white, self.fontDefault)
        swapSeparatorLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        gridLayout.addWidget(swapSeparatorLabel, 2, 3)

        swapTotalLabel = QtWidgets.QLabel('')
        self.setLabel(swapTotalLabel, self.green, self.fontDefault)
        swapTotalLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.systemWidgets['swapTotal'] = swapTotalLabel

        gridLayout.addWidget(swapTotalLabel, 2, 4)

        # ---------------------------------------------------------------------------
        # Temperature

        tempHLayout = QtWidgets.QHBoxLayout()
        tempHLayout.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        valueLabel = QtWidgets.QLabel('')
        self.systemWidgets['label'] = valueLabel
        self.setLabel(valueLabel, self.white, self.fontDefault)

        tempHLayout.addWidget(valueLabel)

        tempIcon = QtWidgets.QLabel()
        tempIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/temp.png'))
        tempIcon.setFixedHeight(24)
        tempIcon.setFixedWidth(24)

        tempHLayout.addWidget(tempIcon)

        tempCurrentValueLabel = QtWidgets.QLabel('')
        self.setLabel(tempCurrentValueLabel, self.white, self.fontDefault)
        self.systemWidgets['current'] = tempCurrentValueLabel
        tempCurrentValueLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        tempHLayout.addWidget(tempCurrentValueLabel)

        # ---------------------------------------------------------------------------
        verticalLayout.addLayout(gridLayout)
        verticalLayout.addLayout(tempHLayout)

        systemGroupBox.setLayout(verticalLayout)
        self.verticalLayout.addWidget(systemGroupBox)

    def displayPartitions(self):
        mntPoints = self.threadSlow.getPartitions()
        diskGroupBox = self.getDefaultGb(self.lang.getStr('disks'))

        verticalLayout = QtWidgets.QVBoxLayout()
        verticalLayout.setSpacing(3)
        # verticalLayout.setAlignment(QtCore.Qt.AlignTop)

        pbFixedWidth = 260
        labelAlignment = QtCore.Qt.AlignRight
        labelDefaultWidth = 60

        # Devices Health
        devices = self.smart.getDevicesHealth()
        devicesGridLayout = QtWidgets.QGridLayout()

        for i, d in enumerate(devices):
            ssdIcon = QtWidgets.QLabel()
            ssdIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/ssd.png'))
            ssdIcon.setFixedSize(20, 20)

            devicesGridLayout.addWidget(ssdIcon, i, 0)

            deviceLabel = QtWidgets.QLabel(d['device'])
            deviceLabel.setFixedWidth(120)
            self.setLabel(deviceLabel, self.white, self.fontDefault)

            devicesGridLayout.addWidget(deviceLabel, i, 1)

            deviceModelLabel = QtWidgets.QLabel(d['model'])
            deviceModelLabel.setFixedWidth(200)
            self.setLabel(deviceModelLabel, self.white, self.fontDefault)

            devicesGridLayout.addWidget(deviceModelLabel, i, 2)

            tempIcon = QtWidgets.QLabel()
            tempIcon.setPixmap(QtGui.QPixmap(f'{self.config.resource_path}/images/temp.png'))
            tempIcon.setFixedHeight(20)
            tempIcon.setFixedWidth(20)

            devicesGridLayout.addWidget(tempIcon, i, 3)

            deviceTempLabel = QtWidgets.QLabel(f"{d['temp']}°{d['scale']}")
            self.setLabel(deviceTempLabel, self.white, self.fontDefault)
            deviceTempLabel.setAlignment(QtCore.Qt.AlignRight)

            devicesGridLayout.addWidget(deviceTempLabel, i, 4)

            self.diskWidgets.append(
                {
                    'device': deviceLabel,
                    'model': deviceModelLabel,
                    'temp': deviceTempLabel
                }
            )

        verticalLayout.addLayout(devicesGridLayout)

        for mntPoint in mntPoints:
            mntPointGridLayout = QtWidgets.QGridLayout()
            # ------------- mountpoint ----------------------
            mountpointValueLabel = QtWidgets.QLabel(mntPoint['mountpoint'])
            self.setLabel(mountpointValueLabel, self.white, self.fontDefault)

            mntPointGridLayout.addWidget(mountpointValueLabel, 0, 0)

            totalValueLabel = QtWidgets.QLabel(mntPoint['total'])
            self.setLabel(totalValueLabel, self.white, self.fontDefault)
            totalValueLabel.setStyleSheet(self.white)
            totalValueLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

            mntPointGridLayout.addWidget(totalValueLabel, 0, 2)

            verticalLayout.addLayout(mntPointGridLayout)
            # ----------------------------------------------------------
            # used stats

            usedLayout = QtWidgets.QGridLayout()

            usedLabel = QtWidgets.QLabel(f"{self.lang.getStr('used')}:")
            self.setLabel(usedLabel, self.red, self.fontDefault)
            usedLabel.setFixedWidth(labelDefaultWidth)

            usedLayout.addWidget(usedLabel, 0, 0)

            # ProgressBar
            usedPB = QtWidgets.QProgressBar()
            usedPB.setFixedHeight(self.pbDefaultHeight)
            usedPB.setFixedWidth(pbFixedWidth)
            usedPB.setFont(self.fontDefault)
            usedPB.setStyleSheet(self.redPBStyle)
            usedPB.setValue(mntPoint['percentUsed'])

            usedLayout.addWidget(usedPB, 0, 1)

            usedValueLabel = QtWidgets.QLabel(mntPoint['used'])
            self.setLabel(usedValueLabel, self.white, self.fontDefault)
            usedValueLabel.setStyleSheet(self.red)
            usedValueLabel.setAlignment(labelAlignment)

            usedLayout.addWidget(usedValueLabel, 0, 2)

            verticalLayout.addLayout(usedLayout)

            # ----------------------------------------------------------
            # free stats

            freeLayout = QtWidgets.QGridLayout()

            freeLabel = QtWidgets.QLabel(f"{self.lang.getStr('free')}:")
            self.setLabel(freeLabel, self.green, self.fontDefault)
            freeLabel.setFixedWidth(labelDefaultWidth)

            freeLayout.addWidget(freeLabel, 0, 0)

            freePB = QtWidgets.QProgressBar()
            freePB.setFixedHeight(self.pbDefaultHeight)
            freePB.setFont(self.fontDefault)
            freePB.setStyleSheet(self.greenPBStyle)
            freePB.setFixedWidth(pbFixedWidth)
            freePB.setValue(mntPoint['percentFree'])

            freeLayout.addWidget(freePB, 0, 1)

            freeValueLabel = QtWidgets.QLabel(mntPoint['free'])
            self.setLabel(freeValueLabel, self.green, self.fontDefault)
            freeValueLabel.setAlignment(labelAlignment)

            freeLayout.addWidget(freeValueLabel, 0, 2)

            verticalLayout.addLayout(freeLayout)
            # ----------------------------------------------------------

            tempDict = dict()
            tempDict['mountpointValueLabel'] = mountpointValueLabel
            tempDict['totalValueLabel'] = totalValueLabel
            tempDict['usedValueLabel'] = usedValueLabel
            tempDict['usedPB'] = usedPB
            tempDict['freeValueLabel'] = freeValueLabel
            tempDict['freePB'] = freePB
            self.partitionsWidgets.append(tempDict)

        diskGroupBox.setLayout(verticalLayout)
        self.verticalLayout.addWidget(diskGroupBox)

    def loadPosition(self):
        # Adjust initial position
        if self.config.getConfig('position') == 'Top Left':
            self.moveTopLeft()
        else:
            self.moveTopRight()

    def contextMenuEvent(self, event: QtGui.QContextMenuEvent):
        contextMenu = QtWidgets.QMenu(self)
        quitAction = contextMenu.addAction('&Quit')
        action = contextMenu.exec_(self.mapToGlobal(event.pos()))
        if action == quitAction:
            sys.exit()

    def moveTopLeft(self):
        self.move(0, 10)

    def moveTopRight(self):
        screen = self.app.primaryScreen()
        logger.info('{}: {}'.format(self.lang.getStr('screen'), screen.name()))
        size = screen.size()
        logger.info('{} {}: {} x {}'.format(self.lang.getStr('screen'), self.lang.getStr('resolution'), size.width(),
                                            size.height()))
        rect = screen.availableGeometry()
        logger.info('{}: {} x {}'.format(self.lang.getStr('spcavail'), rect.width(), rect.height()))
        # move window to top right
        win = self.geometry()
        self.move(rect.width() - win.width(), 0)

    def receiveThreadSlowFinish(self, message):
        for i, msg in enumerate(message):
            self.partitionsWidgets[i]['mountpointValueLabel'].setText(msg['mountpoint'])
            self.partitionsWidgets[i]['totalValueLabel'].setText(msg['total'])
            self.partitionsWidgets[i]['usedValueLabel'].setText(msg['used'])
            self.partitionsWidgets[i]['usedPB'].setValue(msg['percentUsed'])
            self.partitionsWidgets[i]['freeValueLabel'].setText(msg['free'])
            self.partitionsWidgets[i]['freePB'].setValue(msg['percentFree'])

        if self.config.isOnline():
            ipaddrs = self.threadSlow.getIpAddrs()
            self.systemWidgets['intip'].setText(ipaddrs['intip'])
            self.systemWidgets['extip'].setText(ipaddrs['extip'])

    def receiveThreadFastfinish(self, message):
        self.dtwWidgets['hour'].setText(message['hour'])
        self.dtwWidgets['min'].setText(message['min'])
        # self.dtwWidgets['sec'].setText(message['sec'])
        self.dtwWidgets['day'].setText(f"{message['day']}")
        self.dtwWidgets['month'].setText(f", {message['month']} ")
        self.dtwWidgets['year'].setText(message['year'])
        self.dtwWidgets['weekday'].setText(message['weekday'])
        # --------------------------------------------------------
        # update cpu load
        self.systemWidgets['cpuProgressBar'].setValue(message['cpuProgressBar'])
        self.analizeProgressBar(self.systemWidgets['cpuProgressBar'], message['cpuProgressBar'])
        self.systemWidgets['ramProgressBar'].setValue(message['ramProgressBar'])
        self.analizeProgressBar(self.systemWidgets['ramProgressBar'], message['ramProgressBar'])
        self.systemWidgets['swapProgressBar'].setValue(message['swapProgressBar'])
        self.analizeProgressBar(self.systemWidgets['swapProgressBar'], message['swapProgressBar'])
        # ----------------------------
        # update  cpu temperature
        # print(message)
        self.systemWidgets['label'].setText(message['label'])
        self.systemWidgets['current'].setText(f"{int(message['current'])}°{message['scale']}")

        self.systemWidgets['cpuFreqCurrent'].setText(f"{message['cpuFreqCurrent']} MHz")
        self.analizeValue(self.systemWidgets['cpuFreqCurrent'], message['cpuFreqCurrent'], message['cpuFreqMax'])
        self.systemWidgets['cpuFreqMax'].setText(f"{message['cpuFreqMax']} MHz")

        self.systemWidgets['ramUsed'].setText(f"{message['ramUsed']}")

        self.analizeValue(self.systemWidgets['ramUsed'], message['ramUsed'], message['ramTotal'])
        self.systemWidgets['ramTotal'].setText(f"{message['ramTotal']}")

        self.systemWidgets['swapUsed'].setText(f"{message['swapUsed']}")
        self.analizeValue(self.systemWidgets['swapUsed'], message['swapUsed'], message['swapTotal'])
        self.systemWidgets['swapTotal'].setText(f"{message['swapTotal']}")

        # --------------------------------------------------------------------------------------
        # update boot time
        # self.systemWidgets['boottime'].setText(message['boottime'])
        self.systemWidgets['btDays'].setText(f"{message['btDays']} ")
        self.systemWidgets['btHours'].setText(f"{message['btHours']} ")
        self.systemWidgets['btMinutes'].setText(f"{message['btMinutes']} ")
        self.systemWidgets['btSeconds'].setText(f"{message['btSeconds']} ")

        # --------------------------------------------------------------------------------------
        self.analizeTemp(self.systemWidgets['current'], message['current'], message['high'], message['critical'])

        for i, d in enumerate(message['devices']):
            # print(d)
            self.diskWidgets[i]['device'].setText(d['device'])
            self.diskWidgets[i]['model'].setText(d['model'])
            self.diskWidgets[i]['temp'].setText(f"{int(d['temp'])}°{message['scale']}")
            self.analizeTemp(self.diskWidgets[i]['temp'], d['temp'], d['high'], d['critical'])

    def receiveThreadWeatherFinish(self, message):
        # logger.info(message)
        self.dtwWidgets['temp'].setText(f"{int(message['temp'])}°{message['scale']}")
        self.dtwWidgets['humidity'].setText(message['humidity'])
        self.dtwWidgets['pressure'].setText(message['pressure'])
        self.dtwWidgets['visibility'].setText(message['visibility'])
        self.dtwWidgets['wind'].setText(message['wind'])
        self.dtwWidgets['cloudicon'].setPixmap(message['icon'])

    def receiveThreadNetworkStats(self, message):
        self.upDownRateWidgets[0].setText(message['iface'])
        self.upDownRateWidgets[1].setText('{}/s'.format(humanfriendly.format_size(message['downSpeed'], binary=True)))
        self.upDownRateWidgets[2].setText('{}/s'.format(humanfriendly.format_size(message['upSpeed'], binary=True)))
        self.upDownRateWidgets[3].setText(humanfriendly.format_size(message['bytesRcv'], binary=True))
        self.upDownRateWidgets[4].setText(humanfriendly.format_size(message['bytesSent'], binary=True))

    def receiveThreadRadeon(self, message):
        # print(message)
        for i, msg in enumerate(message):
            self.radeonWidgets[i]['gpu_name'].setText(msg['gpu_name'])
            # self.radeonWidgets[i]['utilization_gpu'].setText(f"{str(msg['utilization_gpu'])}")
            self.radeonWidgets[i]['usedTotalMemory'].setText(f"{msg['memory_used']}/{msg['memory_info']['vram_size']}")
            # self.radeonWidgets[i]['power_draw'].setText(f"{msg['power_draw']}")
            # self.radeonWidgets[i]['fan_speed'].setText(f"{msg['fan_speed']}")
            self.radeonWidgets[i]['temperature_gpu'].setText(
                f"{int(msg['temperature_gpu'])}°{msg['temperature_scale']}")
            self.analizeTemp(self.radeonWidgets[i]['temperature_gpu'], msg['temperature_gpu'],
                             msg['temperature_gpu_high'],
                             msg['temperature_gpu_critical'])

    def receiveThreadNvidia(self, message):
        # print(message)
        for i, msg in enumerate(message):
            self.nvidiaWidgets[i]['gpu_name'].setText(msg['gpu_name'])
            self.nvidiaWidgets[i]['utilization_gpu'].setText(f"{str(msg['utilization_gpu'])}")
            self.nvidiaWidgets[i]['usedTotalMemory'].setText(f"{msg['memory_used']}/{msg['memory_total']}")
            self.nvidiaWidgets[i]['power_draw'].setText(f"{msg['power_draw']}")
            self.nvidiaWidgets[i]['fan_speed'].setText(f"{msg['fan_speed']}")
            self.nvidiaWidgets[i]['temperature_gpu'].setText(
                f"{int(msg['temperature_gpu'])}°{msg['temperature_scale']}")
            self.analizeTemp(self.nvidiaWidgets[i]['temperature_gpu'], msg['temperature_gpu'],
                             msg['temperature_gpu_high'],
                             msg['temperature_gpu_critical'])

    @staticmethod
    def analizeTemp(label, current, highValue, criticalValue):
        colorNormal = 'color: rgb(157, 255, 96);'
        colorWarning = 'color: rgb(255, 255, 153);'
        colorAlarm = 'color: rgb(255, 79, 79);'
        label.setStyleSheet(colorNormal)
        if current >= criticalValue:
            label.setStyleSheet(colorAlarm)
        elif (current < criticalValue) and (current >= highValue):
            label.setStyleSheet(colorWarning)

    def analizeProgressBar(self, pb, value):
        if value < 50:
            pb.setStyleSheet(self.greenPBStyle)
        elif (value >= 50) and (value < 80):
            pb.setStyleSheet(self.yellowPBStyle)
        elif value >= 80:
            pb.setStyleSheet(self.redPBStyle)

    @staticmethod
    def setLabel(label, labelcolor, font):
        label.setFont(font)
        label.setStyleSheet(labelcolor)

    def analizeValue(self, lbl, current, maximun):
        pattern = "[^\d\.]"
        currentValue = float(re.sub(pattern, '', current))
        maxValue = float(re.sub(pattern, '', maximun))
        highValue = maxValue - (maxValue * 0.4)

        if currentValue < highValue:
            lbl.setStyleSheet(self.green)
        elif (currentValue >= highValue) and (currentValue < maxValue):
            lbl.setStyleSheet(self.yellow)
        elif currentValue >= maxValue:
            lbl.setStyleSheet(self.red)
