import os
from pathlib import Path
from PyInquirer import prompt
import psutil
import json
import sys
from cpuinfo import get_cpu_info
import distro
import requests
import subprocess
import netifaces
import urllib.request
import coloredlogs
import logging
from telnetlib import Telnet
import numpy as np
import socket
import platform
import re
import pyamdgpuinfo
import time
from babel import default_locale
from babel.dates import format_date
from datetime import datetime
import i18n
import humanfriendly

os.environ['QT_LOGGING_RULES'] = "qt5ct.debug=false"
logger = logging.getLogger(__name__)
coloredlogs.install()


class Language:

    def __init__(self):
        i18n.set('filename_format', '{locale}.{format}')
        i18n.set('locale', self.getLocalLanguage())
        i18n.load_path.append(f'{os.path.dirname(__file__)}/locale')

    @staticmethod
    def getLocalLanguage():
        localLanguage = default_locale('LC_MESSAGES')
        if localLanguage == '':
            localLanguage = 'en_US'

        return localLanguage

    @staticmethod
    def getStr(string):
        return i18n.t(string)


class Radeon:
    n_devices = pyamdgpuinfo.detect_gpus()
    radeon_entity = dict()

    def __init__(self):
        if self.has_radeon():
            self.radeon_entity.update(
                {
                    'count': int(self.n_devices),
                    'gpus': self.get_radeon_gpus_info()
                }
            )

    def get_radeon_gpus_info(self):
        num_gpus = int(self.n_devices)
        message = list()
        if num_gpus > 0:
            for id in range(num_gpus):
                temp_dict = dict()
                gpu = pyamdgpuinfo.get_gpu(id)
                temp_dict.update({
                    'id': id,
                    'gpu_name': gpu.name,
                    'memory_info': gpu.memory_info,
                    'memory_used': gpu.query_vram_usage(),
                    'memory_free': gpu.memory_info['vram_size'] - gpu.query_vram_usage(),
                    'temperature_gpu': gpu.query_temperature(),
                    'temperature_gpu_high': 70.0,
                    'temperature_gpu_critical': 85.0,  # 40% above
                    'temperature_scale': 'C'
                })
                message.append(temp_dict)
        else:
            message.append(list())

        return message

    def has_radeon(self):
        return True if self.n_devices >= 1 else False

    @staticmethod
    def show():
        first_gpu = pyamdgpuinfo.get_gpu(0)
        return first_gpu


class Nvidia:
    smiCommand = 'nvidia-smi'
    # , nounits
    smiSuffixCommand = '--format=csv,noheader'
    smiStatus = subprocess.getstatusoutput(smiCommand)[0]
    nvidiaEntity = dict()

    def __init__(self):
        if self.getSmiStatus():
            count, DriverVersion = self.getOutputCommand('count,driver_version')
            self.nvidiaEntity.update(
                {
                    'count': int(count),
                    'driver_version': DriverVersion,
                    'gpus': self.getGPUsInfo()
                }
            )

    def getNumberGPUs(self):
        if self.getSmiStatus():
            return int(self.getOutputCommand('count')[0])
        else:
            return 0

    def getGPUsInfo(self):
        numGPUS = int(self.getOutputCommand('count')[0])
        message = list()
        if numGPUS > 0:
            for id in range(numGPUS):
                tempDict = dict()
                gpu_uuid, gpu_name, display_mode, vbios_version, fan_speed, pstate, memory_total, memory_used, memory_free, temperature_gpu, power_management, power_draw, clocks_current_graphics, clocks_current_sm, clocks_current_memory, clocks_current_video, utilization_gpu = self.getOutputCommand(
                    'gpu_uuid,gpu_name,display_mode,vbios_version,fan.speed,pstate,memory.total,memory.used,memory.free,temperature.gpu,power.management,power.draw,clocks.current.graphics,clocks.current.sm,clocks.current.memory,clocks.current.video,utilization.gpu'
                )

                tempDict.update({
                    'id': id,
                    'gpu_uuid': gpu_uuid,
                    'gpu_name': gpu_name,
                    'display_mode': display_mode,
                    'vbios_version': vbios_version,
                    'fan_speed': fan_speed,
                    'pstate': pstate,
                    'memory_total': memory_total,
                    'memory_used': memory_used,
                    'memory_free': memory_free,
                    'temperature_gpu': float(temperature_gpu),
                    'temperature_gpu_high': 70.0,
                    'temperature_gpu_critical': 85.0,  # 40% above
                    'temperature_scale': 'C',
                    'power_management': power_management,
                    'power_draw': power_draw,
                    'clocks_current_graphics': clocks_current_graphics,
                    'clocks_current_sm': clocks_current_sm,
                    'clocks_current_memory': clocks_current_memory,
                    'clocks_current_video': clocks_current_video,
                    'utilization_gpu': utilization_gpu
                })
                message.append(tempDict)

        return message

    def getOutputCommand(self, queryList):
        return subprocess.getoutput(f"{self.smiCommand} --id=0 --query-gpu={queryList} {self.smiSuffixCommand}").split(
            ',')

    def getSmiStatus(self):
        if self.smiStatus == 0:
            return True
        else:
            return False


class VirtualMachine:
    @staticmethod
    def getStatus():
        outCmd = subprocess.getoutput('systemd-detect-virt')
        if 'none' in outCmd:
            return False
        else:
            return True


class Config:
    resource_path = os.path.dirname(__file__)
    distrosDir = f'{resource_path}/images/distros'
    cfgFile = f'{Path.home()}/.config/gonha/config.json'
    globalJSON = dict()
    apiKey = 'at_cY0kTF6KP8LuMrXidniTMnkOa7XTE'
    url = 'https://ip-geolocation.whoisxmlapi.com/api/v1'
    myExtIp = subprocess.getoutput('curl -s ifconfig.me')
    outJson = {'city': None, 'region': None, 'country': None}
    nvidia = Nvidia()
    lang = Language()

    def __init__(self):
        self.version = self.getVersion()
        if not os.path.isfile(self.cfgFile):
            self.wizard()

    def wizard(self):
        # check if config file exists
        if os.path.isfile(self.cfgFile):
            os.remove(self.cfgFile)
        # ----------------------------------------------------------------
        # Get cpu info
        cpuInfo = get_cpu_info()
        # ----------------------------------------------------------------
        # update with current version
        self.updateConfig({'version': self.getVersion()})

        # get Platform especific details
        plat = platform.uname()
        self.updateConfig({
            'platform': {
                'system': plat.system,
                'node': plat.node,
                'release': plat.release,
                'machine': plat.machine
            }
        })
        # ----------------------------------------------------------------
        # update with distro information
        dist = {
            'distro': {
                'id': distro.id(),
                'name': distro.name(),
                'codename': distro.codename(),
                'version': distro.version(),
                'iconfile': f'{self.distrosDir}/{distro.id()}.png'
            }
        }
        self.updateConfig(dist)
        # ----------------------------------------------------------------
        logger.info(self.lang.getStr('startwizard'))
        # ----------------------------------------
        # temperature format Question
        tempTypeQuestions = [
            {
                'type': 'list',
                'name': 'temptype',
                'message': i18n.t('tempquestion'),
                'choices': [
                    'Kelvin',
                    'Fahrenheit',
                    'Celsius'
                ],
            }
        ]
        psutil.sensors_temperatures('kelvin')
        tempTypeResponse = prompt(tempTypeQuestions)
        self.updateConfig(tempTypeResponse)

        # ----------------------------------------------------------------
        # Check if nvmes exists
        nvme = self.getNvmes()
        if len(nvme) >= 1:
            # -----------------------------------------------------------------------------------------------------
            nvmeChoices = list()
            nvmeChoices.append({
                'name': 'device: [{}] [/dev/{}]'.format(nvme[0]['id'], nvme[0]['name']),
                'value': nvme[0]['name']
            })

            nvmeQuestions = [
                {
                    'type': 'checkbox',
                    'name': 'nvme',
                    'message': self.lang.getStr('nvmequestion'),
                    'choices': nvmeChoices,
                }
            ]
            nvmeResponse = prompt(nvmeQuestions)
            self.updateConfig(nvmeResponse)
        else:
            self.updateConfig({'nvme': list()})

        # ---------------------------------------------------------------
        # GPuDialog
        # gpus = GPUtil.getGPUs()
        if self.nvidia.getNumberGPUs() > 0:
            gpuChoices = []
            # Filesystem sections
            for gpu in self.nvidia.nvidiaEntity['gpus']:
                gpuChoices.append(
                    {
                        'name': 'id: [{}] model [{}] uuid: [{}]'.format(gpu['id'], gpu['gpu_name'], gpu['gpu_uuid']),
                        'value': gpu['gpu_uuid']
                    }
                )

            gpuQuestions = [
                {
                    'type': 'checkbox',
                    'name': 'nvidia',
                    'message': self.lang.getStr('nvidiaquestion'),
                    'choices': gpuChoices,
                }
            ]

            gpuResponse = prompt(gpuQuestions)
            self.updateConfig(gpuResponse)
        else:
            self.updateConfig({'nvidia': list()})

        if not self.isOnline():
            logger.info(f"{self.lang.getStr('error')}: {self.lang.getStr('offline')}")
            sys.exit(1)

        logger.info(self.lang.getStr('retlocalization'))
        geoData = self.getWeatherData()
        logger.info(self.lang.getStr('next'))
        geoQuestions = [
            {
                'type': 'input',
                'name': 'city',
                'message': self.lang.getStr('cityname'),
                'default': geoData['city']
            },
            {
                'type': 'input',
                'name': 'region',
                'message': self.lang.getStr('region'),
                'default': geoData['region']
            },
            {
                'type': 'input',
                'name': 'country',
                'message': self.lang.getStr('countrycode'),
                'default': geoData['country']
            }
        ]

        geoResponse = prompt(geoQuestions)
        self.updateConfig({'location': geoResponse})
        # ----------------------------------------
        # Position Question
        positionQuestions = [
            {
                'type': 'list',
                'name': 'position',
                'message': self.lang.getStr('position'),
                'choices': [
                    'Top Left',
                    'Top Right',
                ],
            }
        ]
        positionResponse = prompt(positionQuestions)
        self.updateConfig(positionResponse)
        # -------------------------------------------------------------------------
        # Cpu Info
        self.updateConfig({'cpuinfo': cpuInfo['brand_raw']})

        # -------------------------------------------------------------------------
        # if Inside virtual machine, so bypass
        if not VirtualMachine().getStatus():
            # Temperature Question
            cpuSensors = psutil.sensors_temperatures()
            tempUserChoices = []
            for index, sensor in enumerate(cpuSensors):
                for shwtemp in cpuSensors[sensor]:
                    tempUserChoices.append({
                        'name': '{} - [{}] current temp: {:.0f}°C'.format(index, shwtemp.label, shwtemp.current),
                        'value': {'index': index, 'label': shwtemp.label}
                    })

            tempQuestions = [
                {
                    'type': 'list',
                    'name': 'cputemp',
                    'message': self.lang.getStr('cputempquestion'),
                    'choices': tempUserChoices,
                }
            ]
            tempResponse = prompt(tempQuestions)
            self.updateConfig(tempResponse)
        # -----------------------------------------------------------------------------------------------------
        partitionsChoices = []
        # Filesystem sections
        for partition in psutil.disk_partitions():
            partitionsChoices.append(
                {
                    'name': 'device: [{}] mountpoint: [{}] fstype: {}'.format(partition.device,
                                                                              partition.mountpoint,
                                                                              partition.fstype),
                    'value': partition.mountpoint
                }
            )

        partitionQuestions = [
            {
                'type': 'checkbox',
                'name': 'filesystems',
                'message': self.lang.getStr('partquestion'),
                'choices': partitionsChoices,
            }
        ]
        partitionsResponse = prompt(partitionQuestions)
        self.updateConfig(partitionsResponse)
        # -----------------------------------------------------------------------------------------------------

        # Interface Name
        ifaceChoices = []
        for net_if_addr in psutil.net_if_addrs():
            ifaceChoices.append('{}'.format(net_if_addr))

        ifaceQuestions = [
            {
                'type': 'list',
                'name': 'iface',
                'message': self.lang.getStr('ifacequestion'),
                'choices': ifaceChoices
            }
        ]
        ifaceResponse = prompt(ifaceQuestions)
        self.updateConfig(ifaceResponse)

        # Write json global
        # logger.info(self.globalJSON)
        self.writeConfig()

        logger.info(self.lang.getStr('thatsok'))
        logger.info(self.lang.getStr('nowrungonha'))
        # ----------------------------------------
        sys.exit()

    def getConfig(self, key):
        with open(self.cfgFile, 'r') as openfile:
            json_object = json.load(openfile)

        return json_object[key]

    def writeConfig(self):
        if not os.path.isdir(os.path.dirname(self.cfgFile)):
            os.makedirs(os.path.dirname(self.cfgFile))

        # Serializing json
        json_object = json.dumps(self.globalJSON, indent=4)
        with open(self.cfgFile, 'w') as outfile:
            outfile.write(json_object)

    def updateConfig(self, data):
        self.globalJSON.update(data)

    @staticmethod
    def getVersion():
        return '1.7.4'

    def getExtIp(self):
        return self.myExtIp

    def getIntIp(self):
        eth = netifaces.ifaddresses(self.getConfig('iface'))
        return eth[netifaces.AF_INET][0]['addr']

    @staticmethod
    def getGw():
        gws = netifaces.gateways()
        return gws['default'][netifaces.AF_INET][0]

    @staticmethod
    def convertToFahrenheit(temp):
        return (temp * 1.8) + 32

    @staticmethod
    def convertToKelvin(temp):
        return 273.15 + temp

    def normalizeTemps(self, current, high, critical):
        tempConfig = self.getConfig('temptype')
        retCurrent = current
        retHigh = high
        retCritical = critical
        retScale = 'C'
        if tempConfig == 'Kelvin':
            retCurrent = self.convertToKelvin(current)
            retHigh = self.convertToKelvin(high)
            retCritical = self.convertToKelvin(critical)
            retScale = 'K'
        elif tempConfig == 'Fahrenheit':
            retCurrent = self.convertToFahrenheit(current)
            retHigh = self.convertToFahrenheit(high)
            retCritical = self.convertToFahrenheit(critical)
            retScale = 'F'

        return retCurrent, retHigh, retCritical, retScale

    def getWeatherData(self):
        response = requests.get(f"{self.url}?apiKey={self.apiKey}&ipAddress={self.myExtIp}")

        if response.status_code == 200:
            self.outJson.clear()
            tempJson = json.loads(response.text)
            self.outJson.update(
                {
                    'city': tempJson['location']['city'],
                    'region': tempJson['location']['region'],
                    'country': tempJson['location']['country'],
                    'lat': tempJson['location']['lat'],
                    'lng': tempJson['location']['lng']

                }
            )

        return self.outJson

    # Check for nvmes
    @staticmethod
    def getNvmes():
        nvmesJson = json.loads(subprocess.getoutput('lsblk --json'))
        nvmesRet = list()
        for i, nvme in enumerate(nvmesJson['blockdevices']):
            if 'nvme' in nvme['name']:
                tempDict = dict()
                tempDict['id'] = i
                tempDict['name'] = nvme['name']
                nvmesRet.append(tempDict)

        return nvmesRet

    @staticmethod
    def isOnline():
        try:
            socket.create_connection(("8.8.8.8", 53))
            return True
        except OSError:
            return False

    @staticmethod
    def getKernelInfo():
        kernelPattern = "([0-9].[0-9].[0-9]+)"
        kernelString = platform.platform()
        kernelString = re.search(kernelPattern, kernelString).group(0)
        kernelList = kernelString.split('.')
        kernelDict = dict()
        kernelDict.update({'kernelVersion': int(kernelList[0]), 'majorRevision': int(kernelList[1]),
                           'minorRevision': int(kernelList[2])})
        return kernelDict

    @staticmethod
    def getUptime():
        bootTime = time.time() - psutil.boot_time()
        days = bootTime // (24 * 3600)
        bootTime = bootTime % (24 * 3600)
        hours = bootTime // 3600
        bootTime %= 3600
        minutes = bootTime // 60
        bootTime %= 60
        seconds = bootTime
        return [int(days), int(hours), int(minutes), int(seconds)]


class Weather:
    config = Config()
    city = config.getConfig('location')['city']
    url = 'http://api.openweathermap.org/data/2.5/weather?q='
    apikey = 'e943e3d03143693768df6ca7c621c8b5'
    iconUrlPrefix = 'https://openweathermap.org/img/wn/'
    iconUrlSuffix = '@2x.png'

    def getData(self):
        try:
            res = requests.get(f'{self.url}{self.city}&APPID={self.apikey}&units=metric')
            return res.json()
        except Exception as e:
            self.printException(e)

    def getIcon(self, iconStr):
        if self.config.isOnline():
            with urllib.request.urlopen(f"{self.iconUrlPrefix}{iconStr}{self.iconUrlSuffix}") as response:
                return response.read()

    @staticmethod
    def printException(e):
        logger.info(f"{i18n.t('error')}: {e}")


class Smart:
    host = '127.0.0.1'
    port = 7634
    vm = VirtualMachine()
    model = str()
    temp = 0
    message = list()
    storageType = 'sata'
    config = Config()
    temperature = {
        'format': 'Celsius',
        'scale': 'C',
        'current': 0.0,
        'high': 0.0,
        'critical': 0.0
    }

    def __init__(self):
        self.analizeScale()

    def hddtempIsOk(self):
        try:
            socket.create_connection((self.host, self.port))
            return True
        except OSError:
            return False

    def analizeScale(self):
        tempType = self.config.getConfig('temptype')
        if tempType == 'Kelvin':
            self.temperature['format'] = 'Kelvin'
            self.temperature['scale'] = 'K'
        elif tempType == 'Fahrenheit':
            self.temperature['format'] = 'Fahrenheit'
            self.temperature['scale'] = 'F'

    def uniFormTempValues(self, current, high, critical):
        self.temperature['current'] = current
        self.temperature['high'] = high
        self.temperature['critical'] = critical
        if self.temperature['scale'] == 'K':
            self.temperature['current'] = self.config.convertToKelvin(current)
            self.temperature['high'] = self.config.convertToKelvin(high)
            self.temperature['critical'] = self.config.convertToKelvin(critical)

        if self.temperature['scale'] == 'F':
            self.temperature['current'] = self.config.convertToFahrenheit(current)
            self.temperature['high'] = self.config.convertToFahrenheit(high)
            self.temperature['critical'] = self.config.convertToFahrenheit(critical)

    def getDevicesHealth(self):
        self.message.clear()
        self.message = self.getHddTemp()
        devices = self.config.getConfig('nvme')
        if len(devices) >= 1:
            sensors = psutil.sensors_temperatures()
            for sensor in sensors:
                if 'nvme' in sensor:
                    current = sensors[sensor][0].current
                    high = sensors[sensor][0].high
                    if high is None:
                        high = 70.0

                    critical = sensors[sensor][0].critical
                    if critical is None:
                        critical = 82.0

                    self.uniFormTempValues(current, high, critical)

                    self.model = sensors[sensor][0].label
                    self.message.append({
                        'device': '/dev/{}'.format(devices[0]),
                        'model': '{}'.format(devices[0]),
                        'temp': self.temperature['current'],
                        'scale': self.temperature['scale'],
                        'high': self.temperature['high'],
                        'critical': self.temperature['critical'],
                    })

        return self.message

    def getHddTemp(self):
        message = list()
        if self.hddtempIsOk():
            if not self.vm.getStatus():
                with Telnet(self.host, self.port) as tn:
                    lines = tn.read_all().decode('utf-8')

                if lines != '':
                    data = lines
                    # remove first char
                    data = data[1:]
                    # remove the last char
                    data = ''.join([data[i] for i in range(len(data)) if i != len(data) - 1])
                    # replace double || by one |
                    data = data.replace('||', '|')
                    # convert to array
                    data = data.split('|')
                    dataLen = len(data)
                    forLenght = int(dataLen / 4)
                    newarray = np.array_split(data, forLenght)
                    for na in newarray:
                        current = float(na[2])
                        high = current + (current * 0.3)
                        critical = current + (current * 0.4)

                        self.uniFormTempValues(current, high, critical)

                        message.append({
                            'device': na[0],
                            'model': na[1],
                            'temp': self.temperature['current'],
                            'scale': self.temperature['scale'],
                            'high': self.temperature['high'],
                            'critical': self.temperature['critical'],
                        })

            else:
                # Append fake data to virtual machine
                message.append(
                    {
                        'device': '/dev/vmsda',
                        'model': 'VIRTUAL SSD',
                        'temp': 38.0,
                        'high': 70.0,
                        'critical': 80.0,
                        'scale': 'C'
                    }
                )

            return message


class System:
    config = Config()
    message = dict()
    smart = Smart()
    lang = Language()

    def getMessage(self):
        self.message.clear()
        now = datetime.now()
        # print(format_date(now, format='full', locale=self.lang.getLocalLanguage()))

        self.message['hour'] = now.strftime('%H')
        self.message['min'] = now.strftime('%M')
        self.message['sec'] = now.strftime('%S')
        self.message['day'] = format_date(now, format='d', locale=self.lang.getLocalLanguage())
        self.message['weekday'] = format_date(now, format='EEEE', locale=self.lang.getLocalLanguage())
        self.message['month'] = format_date(now, format='MMMM', locale=self.lang.getLocalLanguage())
        self.message['year'] = format_date(now, format='YYYY', locale=self.lang.getLocalLanguage())

        cpuFreq = psutil.cpu_freq()
        ram = psutil.virtual_memory()
        swap = psutil.swap_memory()

        self.message['cpuFreqCurrent'] = '{:.0f}'.format(cpuFreq.current)
        self.message['cpuFreqMax'] = '{:.0f}'.format(cpuFreq.max)

        self.message['ramUsed'] = '{}'.format(humanfriendly.format_size(ram.used, binary=True))
        self.message['ramTotal'] = '{}'.format(humanfriendly.format_size(ram.total, binary=True))

        self.message['swapUsed'] = '{}'.format(humanfriendly.format_size(swap.used, binary=True))
        self.message['swapTotal'] = '{}'.format(humanfriendly.format_size(swap.total, binary=True))

        self.message['cpuProgressBar'] = psutil.cpu_percent()
        self.message['ramProgressBar'] = ram.percent
        self.message['swapProgressBar'] = swap.percent

        # Get boot time
        btDays, btHours, btMinutes, btSeconds = self.config.getUptime()
        self.message['btDays'] = btDays
        self.message['btHours'] = btHours
        self.message['btMinutes'] = btMinutes
        self.message['btSeconds'] = btSeconds

        # --------------------------------------------------------
        # if inside virtual machine , so bypass sensor
        if not VirtualMachine().getStatus():
            tempConfig = self.config.getConfig('cputemp')
            sensors = psutil.sensors_temperatures()
            for i, sensor in enumerate(sensors):
                if i == tempConfig['index']:
                    for shwtemp in sensors[sensor]:
                        if shwtemp.label == tempConfig['label']:
                            current = shwtemp.current
                            high = shwtemp.high
                            critical = shwtemp.critical
                            if high is None:
                                high = 75.0  # 75 celsius for high temp cpu

                            if critical is None:
                                critical = 90.0  # 90 celsius for critical temp cpu

                            current, high, critical, scale = self.config.normalizeTemps(current, high, critical)

                            self.message['label'] = shwtemp.label
                            self.message['current'] = current
                            self.message['high'] = high
                            self.message['critical'] = critical
                            self.message['scale'] = '{}'.format(scale)
                            break
        else:
            self.message['label'] = 'vmTdie'
            self.message['current'] = 50.0
            self.message['high'] = 70.0
            self.message['critical'] = 90.0
            self.message['scale'] = 'C'

        # Storages
        devices = self.smart.getDevicesHealth()
        self.message['devices'] = list()
        for d in devices:
            tempDict = dict()
            tempDict['device'] = d['device']
            tempDict['model'] = d['model']
            tempDict['temp'] = d['temp']
            tempDict['high'] = d['high']
            tempDict['critical'] = d['critical']
            self.message['devices'].append(tempDict)

        return self.message
